@@ -1,4 +1,5 @@
 #include <Rcpp.h>
+#include <fstream>
 
 #define RNIFTI_NIFTILIB_VERSION 2
 #include "RNifti.h"
@@ -106,7 +107,7 @@ BEGIN_RCPP
     for (size_t i=0; i<len; i++)
     {
         rgba.value.packed = int(data[i]);
-        for (int j=0; j<channels.size(); j++)
+        for (unsigned j=0; j<channels.size(); j++)
         {
             const int channel = channels[j] - 1;
             result[i + len * j] = int(rgba.value.bytes[channel]);
@@ -173,7 +174,7 @@ BEGIN_RCPP
 END_RCPP
 }
 
-RcppExport SEXP readNiftiBlob (SEXP _file, SEXP _length, SEXP _datatype, SEXP _offset)
+RcppExport SEXP readNiftiBlob (SEXP _file, SEXP _length, SEXP _datatype, SEXP _offset, SEXP _gzipped, SEXP _swap)
 {
 BEGIN_RCPP
     int datatype;
@@ -185,12 +186,33 @@ BEGIN_RCPP
     const std::string filename = as<std::string>(_file);
     const size_t length = as<size_t>(_length);
     const size_t offset = Rf_isNull(_offset) ? 0 : as<size_t>(_offset);
+    const bool swap = as<bool>(_swap);
     
     int nbyper;
     nifti_datatype_sizes(datatype, &nbyper, NULL);
     
-    const bool gzExtension = filename.length() > 3 && filename.substr(filename.length()-3,3) == ".gz";
-    znzFile file = znzopen(filename.c_str(), "rb", gzExtension);
+    // NA means the caller wants us to guess if the file is gzipped
+    // NB: as<bool> gives true for NA_LOGICAL, so we need to convert to int to test for this value
+    bool gzipped = as<bool>(_gzipped);
+    if (as<int>(_gzipped) == NA_LOGICAL)
+    {
+        // If the filename has a ".gz" extension, assume it's gzipped
+        // Otherwise look for the gzip magic number in the first two bytes of the file, possibly swapped
+        if (filename.length() > 3 && filename.substr(filename.length()-3,3) == ".gz")
+            gzipped = true;
+        else
+        {
+            uint16_t magic;
+            std::ifstream stream(filename.c_str(), std::ios::binary);
+            if (stream.fail())
+                Rf_error("Failed to open file %s", filename.c_str());
+            stream.read(reinterpret_cast<char*>(&magic), 2);
+            gzipped = (magic == (swap ? 0x8b1f : 0x1f8b));
+            stream.close();
+        }
+    }
+    
+    znzFile file = znzopen(filename.c_str(), "rb", gzipped);
     if (znz_isnull(file))
         Rf_error("Failed to open file %s", filename.c_str());
     if (offset > 0)
@@ -199,6 +221,9 @@ BEGIN_RCPP
     znzread(buffer, nbyper, length, file);
     znzclose(file);
     
+    if (swap)
+        nifti_swap_Nbytes(length, nbyper, buffer);
+    
     NiftiImageData data(buffer, length, datatype);
     RObject result;
     if (data.isComplex())
@@ -413,7 +438,10 @@ BEGIN_RCPP
         if (image.isNull())
             matrix.attr("code") = 0;
         else
+        {
+            matrix.attr("imagedim") = image.dim();
             matrix.attr("code") = ((preferQuaternion && image->qform_code > 0) || image->sform_code <= 0) ? image->qform_code : image->sform_code;
+        }
         return matrix;
     }
 END_RCPP
@@ -518,16 +546,36 @@ RcppExport SEXP setOrientation (SEXP _image, SEXP _axes)
 BEGIN_RCPP
     if (isXformMatrix(_image))
     {
+        const RObject xform(_image);
+        
         // Create an empty image for temporary purposes
         nifti2_image *ptr = nifti2_make_new_nim(NULL, DT_UNSIGNED_CHAR, 0);
         NiftiImage image(ptr);
         
         // Set the qform matrix
-        image.qform() = NiftiImage::Xform(_image);
+        image.qform() = xform;
         image->qform_code = 2;
         
+        // If the matrix has an image dimension attribute, apply it
+        // If not, warn that the translation component of the xform will be unreliable
+        if (xform.hasAttribute("imagedim"))
+        {
+            const std::vector<NiftiImage::dim_t> dimVector = xform.attr("imagedim");
+            const int nDims = std::min(7, int(dimVector.size()));
+            image->dim[0] = nDims;
+            for (int i=0; i<nDims; i++)
+                image->dim[i+1] = dimVector[i];
+        }
+        else
+            Rf_warning("The origin of a bare xform matrix cannot be reliably preserved through reorientation");
+        
+        // Perform the full reorientation process
         image.reorient(as<std::string>(_axes));
-        return image.qform().matrix();
+        
+        NumericMatrix result = wrap(image.qform().matrix());
+        result.attr("imagedim") = image.dim();
+        result.attr("code") = xform.attr("code");
+        return result;
     }
     else
     {
@@ -786,7 +834,7 @@ R_CallMethodDef callMethods[] = {
     { "asNifti",        (DL_FUNC) &asNifti,         4 },
     { "niftiVersion",   (DL_FUNC) &niftiVersion,    1 },
     { "readNifti",      (DL_FUNC) &readNifti,       3 },
-    { "readNiftiBlob",  (DL_FUNC) &readNiftiBlob,   4 },
+    { "readNiftiBlob",  (DL_FUNC) &readNiftiBlob,   6 },
     { "writeNifti",     (DL_FUNC) &writeNifti,      4 },
     { "niftiHeader",    (DL_FUNC) &niftiHeader,     1 },
     { "analyzeHeader",  (DL_FUNC) &analyzeHeader,   1 },
